/* START_LICENSE_HEADER

Copyright (C) 2000 Martin Piper, original design and program code
Copyright (C) 2001-2005 Replica Software

This program file is copyright (C) Replica Software and can only be used under license.
For more information visit: http://www.replicanet.com/
Or email: info@replicanet.com

END_LICENSE_HEADER */
#ifndef _PATCHER_H_
#define _PATCHER_H_
#include "RNPlatform/Inc/DLLExportAPI.h"

namespace RNReplicaNet
{

namespace RNLobby
{

class PatcherPrivate;

/// This class generates minimal patches for binary files and also patches files to produce new versions.
/// No checking of file integrity is carried out before a patch is applied because this is a file version problem which is
/// the domain of another set of support classes.
class Patcher
{
public:
	Patcher();
	virtual ~Patcher();

	/// Generates a patch from sourcePath compared with newPath and stores the result in patchPath.
	/// \param sourcePath The original file path.
	/// \param newPath The new file path.
	/// \param patchPath The destination file path for the patch file.
	/// \return The length of the generated patch file. This can be zero and indicates the two files are identical. A negative value indicates an error occured.
	int GeneratePatch(const char *sourcePath,const char *newPath,const char *patchPath);

	/// Combines a file from sourcePath with a patch file from patchPath and stores the resultant file in destinationPath.
	/// \param sourcePath The original file path.
	/// \param patchPath The patch file path.
	/// \param destinationPath The destination file path for the combined file.
	/// \return Success returns true. False indicates an error occurred.
	bool PatchFile(const char *sourcePath,const char *patchPath,const char *destinationPath);

private:
	PatcherPrivate &mData;
};

};

};

#endif
