/* START_LICENSE_HEADER

Copyright (C) 2000 Martin Piper, original design and program code
Copyright (C) 2001-2005 Replica Software

This program file is copyright (C) Replica Software and can only be used under license.
For more information visit: http://www.replicanet.com/
Or email: info@replicanet.com

END_LICENSE_HEADER */
#ifndef __ENCRYPTION_H__
#define __ENCRYPTION_H__
#include "RNPlatform/Inc/DLLExportAPI.h"

namespace RNReplicaNet
{

// Key length must be an integer multiple of 4 and be at least 8
const int kEncryptionKeyLengthBytes = 8;

/**
 * An encryption class that is designed to be fast and maintain the size of the original data.
 */
class REPNETEXPORTAPI Encryption
{
public:
	struct REPNETEXPORTAPI Key
	{
		public:
		Key();
		virtual ~Key();

		/**
		 * Creates a key from data and an input length.
		 * \param data the data to use as a seed for the key. This can be NULL to use the default key.
		 * \param length the length of the data to use for creating the key
		 */
		void Create(void *data,int length);

		unsigned char mKey[kEncryptionKeyLengthBytes];
	};

	/**
	 * Encrypt a portion of memory with a key.
	 * \param data the pointer to the start of the memory
	 * \param length the length of the memory in bytes
	 * \param key the key to use
	 */
	static void Encrypt(void *data,int length,Key *key);

	/**
	 * Decrypt a portion of memory with a key.
	 * \param data the pointer to the start of the memory
	 * \param length the length of the memory in bytes
	 * \param key the key to use
	 */
	static void Decrypt(void *data,int length,Key *key);
};

} // namespace RNReplicaNet

#endif
