/* START_LICENSE_HEADER

Copyright (C) 2000 Martin Piper, original design and program code
Copyright (C) 2001-2005 Replica Software

This program file is copyright (C) Replica Software and can only be used under license.
For more information visit: http://www.replicanet.com/
Or email: info@replicanet.com

END_LICENSE_HEADER */
#ifndef __MESSAGEHELPER_H__
#define __MESSAGEHELPER_H__
#include "RNPlatform/Inc/DLLExportAPI.h"
#include <string>

namespace RNReplicaNet
{

#define MESSAGEHELPER_ADDVARIABLE(x)	\
AddVariable(&(x),sizeof(x));

#define MESSAGEHELPER_GETVARIABLE(x)	\
GetVariable(&(x),sizeof(x));

#define MESSAGEHELPER_ADDVARIABLEp(y,x)	\
y.AddVariable(&(x),sizeof(x));

#define MESSAGEHELPER_GETVARIABLEp(y,x)	\
y.GetVariable(&(x),sizeof(x));

/**
 * A simple message helper class that helps to construct and deconstruct sequences of variables.
 * This routine is endian safe and should be used where data needs to be endian safe
 */
class REPNETEXPORTAPI MessageHelper
{
public:
	/**
	 * The ctor performs some basic initialisation
	 */
	MessageHelper();

	/**
	 * The dtor makes sure everything is tidy
	 */
	virtual ~MessageHelper();

	/**
	 * Sets the data buffer pointer to use while constructing or deconstructing this message. This also causes the length of the buffer to revert to zero.
	 * \param buffer the buffer pointer to set
	 */
	void SetBuffer(void *const buffer);

	/**
	 * Adds a variable of a specified size to the message buffer while using an endian check to preserve the value between platforms with different endian formats
	 * \param variable the pointer to the variable
	 * \param size the size of the data
	 */
	void AddVariable(const void *variable,const int size);

	/**
	 * Adds a data block of a specified size to the message buffer without using any endian checking
	 * \param data the pointer to the data
	 * \param size the size of the data
	 */
	void AddData(const void *data,const int size);

	/**
	 * Gets a variable of a specified size from the message buffer
	 * \param variable the pointer to the variable
	 * \param size the size of the data
	 */
	void GetVariable(void *const variable,const int size);

	/**
	 * Gets data of a specified size from the message buffer
	 * \param data the pointer to the data
	 * \param size the size of the data
	 */
	void GetData(void *const data,const int size);

	/**
	 * Returns the current size of the message in bytes
	 * \return returns the size of the message buffer in bytes
	 */
	int GetSize(void);

	/**
	 * Sets the current size of the message in bytes
	 * \param the size of the message in bytes
	 */
	void SetSize(const int size);

	/**
	 * Adds an integer to the message buffer
	 * \param value the value to add
	 */
	void AddInteger(const int value);

	/**
	 * Adds a float to the message buffer
	 * \param value the value to add
	 */
	void AddFloat(const float value);

	/**
	 * Gets an integer from the message buffer
	 * \param value the value to get
	 */
	int GetInteger(void);

	/**
	 * 
	 * Gets a float from the message buffer
	 * \param value the value to get
	 */
	float GetFloat(void);

	/**
	 * This produces a hex dump of any length of data
	 */
	static std::string DumpAsHex(const void *data,const int size);

private:
	unsigned char *mFirstPtr;			/**< The data buffer pointer */
	unsigned char *mBufferPtr;			/**< Backup of the pointer passed in used to calculate size */
};

} // namespace RNReplicaNet

#endif
