/* START_LICENSE_HEADER

Copyright (C) 2000 Martin Piper, original design and program code
Copyright (C) 2001-2005 Replica Software

This program file is copyright (C) Replica Software and can only be used under license.
For more information visit: http://www.replicanet.com/
Or email: info@replicanet.com

END_LICENSE_HEADER */
#ifndef __REGISTRYMANAGER_H__
#define __REGISTRYMANAGER_H__
#include "RNPlatform/Inc/DLLExportAPI.h"

#include <assert.h>

#include <vector>

namespace RNReplicaNet
{

/**
 * RegistryManager is a template class for handling classes in organised vector arrays
 */
template <class T> class RegistryManager
{
public:
	/**
	 * The ctor performs some basic initialisation
	 */
	RegistryManager()
	{
		mUsed = false;
	}

	/**
	 * The dtor makes sure everything is tidy
	 */
	virtual ~RegistryManager()
	{
	}

	/**
	 * Adds an item to the list. This does not allocate an item as this is left to the user to do
	 * \param item the item pointer to add
	 */
	void AddItem(T *item)
	{
		mItems.push_back(item);
	}

	/**
	 * Removes an item from the list but does not free the item as this is left to the user to do
	 * \param item the item pointer to remove
	 */
	void RemoveItem(T *item)
	{
		std::vector<T *>::iterator tst,ten;

		tst = mItems.begin();
		ten = mItems.end();

		while(tst != ten)
		{
			T *titem = (T *) *tst;
			if (titem == item)
			{
				if (mUsed)
				{
					if (tst < st)
					{
						st--;
					}
				}
				mItems.erase(tst);
				return;
			}
			tst++;
		}
	}

	/**
	 * Removes the current iterated item
	 */
	void RemoveItem(void)
	{
		assert(mUsed != false && "RegistryManager::RemoveItem() used when BeginIterate() not used or Iterate() reached the end of the list");

		std::vector<T *>::iterator tst;

		st--;
		tst = st;
		st++;

		mItems.erase(tst);
	}

	/**
	 * Starts the iteration of the items in the list
	 */
	void BeginIterate(void)
	{
		mUsed = true;
		st = mItems.begin();
	}

	/**
	 * Iterates through the items in the list
	 * \return returns the next item pointer in the list or returns NULL for the end of the list
	 */
	T *Iterate(void)
	{
		assert(mUsed != false && "RegistryManager::Iterate() used when BeginIterate() not used or Iterate() reached the end of the list");

		if (st < mItems.end())
		{
			T *tclass = *st++;
			return tclass;
		}

		mUsed = false;
		return 0;
	}

private:
	std::vector<T *> mItems;			/**< The items list */

	typename std::vector<T *>::iterator st,en;	/**< Start and end list iterators */
	bool mUsed;
};

} // namespace RNReplicaNet

#endif
