/* START_LICENSE_HEADER

Copyright (C) 2000 Martin Piper, original design and program code
Copyright (C) 2001-2005 Replica Software

This program file is copyright (C) Replica Software and can only be used under license.
For more information visit: http://www.replicanet.com/
Or email: info@replicanet.com

END_LICENSE_HEADER */
#ifndef __THREAD_H__
#define __THREAD_H__
#include "RNPlatform/Inc/DLLExportAPI.h"

#if defined(linux) || defined(unix)
#include <pthread.h>
#endif

namespace RNReplicaNet
{

class ThreadClass;

/**
 * This Thread class handles creating a class that can be automatically threaded.
 * A threaded class inherits methods from ThreadClass.
 */
class REPNETEXPORTAPI Thread
{
public:
	/**
	 * The ctor performs some basic initialisation
	 */
	Thread();

	/**
	 * The dtor makes sure everything is tidy
	 */
	virtual ~Thread();

	/**
	 * This begins the thread in the specified ThreadClass class.
	 * Execution begins from ThreadClass::ThreadEntry()
	 * \param threaded_class the pointer to the class that inherits from ThreadClass
	 */
	void Begin(ThreadClass *threaded_class);

	/**
	 * Terminates the thread of the class bound to this Thread. The caller must not be in the context of the running thread.
	 */
	void Terminate(void);

	/**
	 * This is a static method where the class starts its thread from
	 * \param thread_class the threaded class pointer
	 */
	static int CallThreadEntry(ThreadClass *thread_class);

	/**
	 * Gets the execution status of the thread
	 * \return return true if the thread is running
	 */
	bool GetIsRunning(void) const;


private:
	ThreadClass *mThreadClass;
#if defined(_WIN32)
	void *mThreadHandle;
//	unsigned long mThreadHandle;
#endif
#if defined(linux) || defined(unix)
	pthread_t mThreadHandle;
#endif
#if defined (_PS2)
	int mThreadHandle;
#endif

	volatile bool mIsRunning;
};

} // namespace RNReplicaNet

#endif
